@@ -24,14 +24,35 @@ const Lang = imports.lang;
 const Utils = imports.utils;
 
 const Columns = {
-    TEXT: 0
+    ICON: 0,
+    TEXT: 1
 };
 
 const SearchPopup = new Lang.Class({
     Name: 'SearchPopup',
     Extends: Gtk.Bin,
 
-    _init: function() {
+    _init: function(numVisible) {
+        this._numVisible = numVisible;
+
+        let ui = Utils.getUIObject('search-popup', ['frame',
+                                                    'scrolled-window',
+                                                    'stack',
+                                                    'spinner',
+                                                    'treeview']);
+
+        this._stack = ui.stack;
+        this._scrolledWindow = ui.scrolledWindow;
+        this._spinner = ui.spinner;
+        this._treeView = ui.treeview;
+
+        this._treeView.connect('button-press-event',
+                               this._onListButtonPress.bind(this));
+        this._initList();
+
+        this.height_request = this._cellHeight * this._numVisible;
+        this._scrolledWindow.set_min_content_height(this.height_request);
+
         this.parent({ width_request: 500,
                       halign: Gtk.Align.CENTER,
                       valign: Gtk.Align.START,
@@ -39,27 +60,26 @@ const SearchPopup = new Lang.Class({
                       no_show_all: true,
                       visible: true });
 
-        let ui = Utils.getUIObject('search-popup', ['frame',
-                                                    'treeview',
-                                                    'treeview-selection']);
-        this._treeView = ui.treeview;
-        this._treeViewSelection = ui.treeviewSelection;
-        this._treeView.connect('button-press-event',
-                               this._onListButtonPress.bind(this));
-        this._initList();
-
         this.add(ui.frame);
         this.hide();
     },
 
     _initList: function() {
-        let cell = new Gtk.CellRendererText({ xpad: 16,
-                                              ypad: 8 });
         let column = new Gtk.TreeViewColumn();
 
         this._treeView.append_column(column);
+
+        let cell = new Gtk.CellRendererPixbuf({ xpad: 2 });
+        column.pack_start(cell, false);
+        column.add_attribute(cell, 'pixbuf', Columns.ICON);
+
+        cell = new Gtk.CellRendererText({ xpad: 8,
+                                          ypad: 8 });
         column.pack_start(cell, true);
         column.add_attribute(cell, 'markup', Columns.TEXT);
+
+        this._cellHeight = column.cell_get_size(null)[3];
+        this._cellHeight += cell.get_preferred_height(this._treeView)[0];
     },
 
     _onListButtonPress: function(widget, event) {
@@ -84,6 +104,36 @@ const SearchPopup = new Lang.Class({
         }
     },
 
+    showSpinner: function() {
+        this._spinner.start();
+        this._stack.set_visible_child(this._spinner);
+
+        if (!this.get_visible())
+            this.show();
+    },
+
+    showResult: function() {
+        if (this._spinner.active)
+            this._spinner.stop();
+
+        this._stack.set_visible_child(this._treeView);
+
+        if (!this.get_visible())
+            this.show();
+    },
+
+    vfunc_show: function() {
+        this._treeView.columns_autosize();
+        this.parent();
+    },
+
+    vfunc_hide: function() {
+        if (this._spinner.active)
+            this._spinner.stop();
+
+        this.parent();
+    },
+
     setModel: function(model) {
         this._treeView.set_model(model);
     },
