@@ -31,12 +31,15 @@ const Geocode = imports.gi.GeocodeGlib;
 const GdkPixbuf = imports.gi.GdkPixbuf;
 const Clutter = imports.gi.Clutter;
 const Cogl = imports.gi.Cogl;
+const Soup = imports.gi.Soup;
 
 const _ = imports.gettext.gettext;
 
 let debugInit = false;
 let debugEnabled = false;
 
+let _iconStore = {};
+
 function debug(str) {
     if (!debugInit) {
         let env = GLib.getenv('MAPS_DEBUG');
@@ -130,3 +133,86 @@ function getUIObject(res, ids) {
     });
     return ret;
 }
+
+function load_icon(icon, size, loadCompleteCallback) {
+    if (icon instanceof Gio.FileIcon) {
+        _load_file_icon(icon, loadCompleteCallback);
+    } else if (icon instanceof Gio.ThemedIcon) {
+        _load_themed_icon(icon, size, loadCompleteCallback);
+    }
+}
+
+function _load_file_icon(icon, loadCompleteCallback) {
+    let pixbuf = _iconStore[icon.file.get_uri()];
+
+    if (pixbuf) { // check if the icon is cached
+        loadCompleteCallback(pixbuf);
+        return;
+    }
+
+    if (icon.file.has_uri_scheme ("http") || icon.file.has_uri_scheme ("https")) {
+        _load_http_icon(icon, loadCompleteCallback);
+        return;
+    }
+
+    icon.load_async(-1, null, function(icon, res) {
+        try {
+            let stream = icon.load_finish(res, null)[0];
+
+            pixbuf =
+                GdkPixbuf.Pixbuf.new_from_stream(stream, null);
+
+            _iconStore[icon.file.get_uri()] = pixbuf;
+            loadCompleteCallback(pixbuf);
+        } catch(e) {
+            log("Failed to load pixbuf: " + e);
+        }
+    });
+}
+
+function _load_http_icon(icon, loadCompleteCallback) {
+    let msg = Soup.form_request_new_from_hash('GET', icon.file.get_uri(), {});
+    let soup_session = _get_soup_session();
+
+    soup_session.queue_message(msg, function(session, msg) {
+        if (msg.status_code != Soup.KnownStatusCode.OK) {
+            log("Failed to load pixbuf: " + msg.reason_phrase);
+            return;
+        }
+
+        let contents = msg.response_body.flatten().get_as_bytes();
+        let stream = Gio.MemoryInputStream.new_from_data
+                    (contents.get_data (null));
+        try {
+            let pixbuf = GdkPixbuf.Pixbuf.new_from_stream(stream, null);
+
+            _iconStore[icon.file.get_uri()] = pixbuf;
+            loadCompleteCallback(pixbuf);
+        } catch(e) {
+            log("Failed to load pixbuf: " + e);
+        }
+    });
+}
+
+let soup_session = null;
+function _get_soup_session() {
+    if (soup_session === null) {
+        log ("creating soup");
+        soup_session = new Soup.Session ();
+    }
+
+    return soup_session;
+}
+
+function _load_themed_icon(icon, size, loadCompleteCallback) {
+    let theme = Gtk.IconTheme.get_default();
+    let flags = Gtk.IconLookupFlags.GENERIC_FALLBACK;
+    let info = theme.lookup_by_gicon(icon, size, flags);
+
+    try {
+        let pixbuf = info.load_icon();
+        loadCompleteCallback(pixbuf);
+    } catch(e) {
+        log("Failed to load pixbuf: " + e);
+    }
+}
